package de.privacynerd.positions;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.Location;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.configuration.InvalidConfigurationException;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.entity.ArmorStand;
import org.bukkit.entity.Player;
import org.bukkit.plugin.java.JavaPlugin;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

class PositionCommand implements CommandExecutor {

    @Override
    public boolean onCommand(CommandSender sender, Command command, String label, String[] args) {
        YamlConfiguration positions = new YamlConfiguration();
        Player player = (Player) sender;
        try {
            positions.load("positions.yaml");
        } catch (IOException e) {
            try {
                new File("positions.yaml").createNewFile();
                player.sendMessage(Positions.PREFIX + ChatColor.RED + "An internal error occurred. Please try to execute the command again.");
                return true;
            } catch (IOException ioException) {
                ioException.printStackTrace();
            }
        } catch (InvalidConfigurationException e) {
            new File("positions.yaml").delete();
            try {
                new File("positions.yaml").createNewFile();

                player.sendMessage(Positions.PREFIX + ChatColor.RED + "An internal error occurred. Please try to execute the command again.");
                return true;
            } catch (IOException ioException) {
                ioException.printStackTrace();
            }
        }
        List<String> positionList = (List<String>) positions.getList("positions");
        if(positionList == null) {
            List<String> list = new ArrayList<>();
            positions.set("positions", list);
            try {positions.save("positions.yaml");} catch (IOException e) {e.printStackTrace();}  // save the configuration
            positionList = (List<String>) positions.getList("positions");
        }

        if(args.length == 0) {player.sendMessage(Positions.PREFIX + ChatColor.RED + "You have to specify at least one argument"); return true;}

        if(args[0].equalsIgnoreCase("list")) {
            if(positionList.isEmpty()) {
                player.sendMessage(Positions.PREFIX + ChatColor.GOLD + "You haven't set any positions to display here!");
            } else {
                player.sendMessage(Positions.PREFIX + ChatColor.GREEN + "Available positions:");
                for(String name : positionList) {
                    Location pos = positions.getLocation(name);
                    int x = pos.getBlockX();
                    int y = pos.getBlockY();
                    int z = pos.getBlockZ();
                    player.sendMessage(ChatColor.GOLD + name + ":  " + ChatColor.RED + x + ChatColor.DARK_GREEN + "  " + y + ChatColor.BLUE + "  " + z);
                }
                player.sendMessage(Positions.PREFIX + ChatColor.ITALIC + "Type \"/pos <name> del>\" to delete them.");
            }
            return true;
        }
        /*if(args[0].equalsIgnoreCase("show")) {
            ArmorStand armorStand = player.getWorld().spawn(player.getLocation(), ArmorStand.class);
            armorStand.setVisible(false);
            armorStand.setCustomName(ChatColor.GOLD + "POSITION");
            armorStand.setCustomNameVisible(true);
        }
        if(args[0].equalsIgnoreCase("hide")) {

        }*/

        //if(!args[0].equalsIgnoreCase("list") && !args[0].equalsIgnoreCase("show") && !args[0].equalsIgnoreCase("hide")) {
        if(!args[0].equalsIgnoreCase("list")) {  // only save a new position when it's not named list (cause that's another action)
            if(positionList.contains(args[0])) {  // does the position already exist?
                if(args[0].equalsIgnoreCase("positions")) {
                    player.sendMessage(Positions.PREFIX + ChatColor.RED + "You can't set (or get) the position \"positions\"");
                    return true;
                } else if(args.length==2 && args[1].startsWith("del")) {
                    positionList.remove(args[0]);
                    positions.set("positions", positionList);
                    try {positions.save("positions.yaml");} catch (IOException e) {e.printStackTrace();}  // save the configuration
                    player.sendMessage(Positions.PREFIX + ChatColor.GREEN + "Successfully deleted the position \"" + ChatColor.BLUE + args[0] + ChatColor.GREEN + "\"!");
                    return true;
                } else {
                    Location pos = positions.getLocation(args[0]);
                    int x = pos.getBlockX();
                    int y = pos.getBlockY();
                    int z = pos.getBlockZ();
                    player.sendMessage(Positions.PREFIX + ChatColor.AQUA + "Position \"" + ChatColor.BLUE + args[0] + ChatColor.AQUA + "\": " + ChatColor.RED + x + ChatColor.GREEN + "  " + y + ChatColor.BLUE + "  " + z);
                    return true;
                }
            } else {
                if(args[0].equalsIgnoreCase("positions")) {
                    player.sendMessage(Positions.PREFIX + ChatColor.RED + "You can't set (or get) the position \"positions\"");
                    return true;
                } else {
                    Location playerPos = player.getLocation();
                    positionList.add(args[0]);
                    positions.set(args[0], playerPos);
                    positions.set("positions", positionList);
                    try {positions.save("positions.yaml");} catch (IOException e) {e.printStackTrace();}  // save the configuration
                    player.sendMessage(Positions.PREFIX + ChatColor.GREEN + "Position \"" + ChatColor.BLUE + args[0] + ChatColor.GREEN + "\" has been saved.");
                    return true;
                }
            }
        }
        return true;
    }
}

public final class Positions extends JavaPlugin {
    public static String PREFIX = ChatColor.BLUE + "[" + ChatColor.LIGHT_PURPLE + "Positions" + ChatColor.BLUE + "] " + ChatColor.WHITE;
    public static Positions INSTANCE;
    public Positions() {
        INSTANCE = this;
    }

    public void log(String msg) {
        Bukkit.getConsoleSender().sendMessage(PREFIX + msg);
    }

    public void chat(String msg) {
        Bukkit.getConsoleSender().sendMessage(PREFIX + msg);
    }

    @Override
    public void onEnable() {
        // Plugin startup logic
        log(ChatColor.GREEN + "Plugin has been enabled");

        Bukkit.getPluginCommand("position").setExecutor(new PositionCommand());
    }

    @Override
    public void onDisable() {
        // Plugin shutdown logic
        log(ChatColor.RED + "Plugin has been disabled");
    }
}
